@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 Defines the function :func:`repr2`, which allows for a bit more customization
 than :func:`repr` or :func:`pprint`. See the docstring for more details.
@@ -39,28 +38,20 @@ extensions keyword argument (although this will be a global change).
 >>> print(ub.repr2({1: float('nan'), 2: float('inf'), 3: 3.0}, nl=0, extensions=extensions))
 {1: monkey(nan), 2: monkey(inf), 3: monkey(3.0)}
 
+As of ubelt 1.1.0 you can now access and update the default extensions via the
+repr2 function itself.
+
+>>> # xdoctest: +SKIP
+>>> # We skip this at test time to not modify global state
+>>> @ub.repr2.EXTENSIONS.register(float)
+>>> def my_float_formater(data, **kw):
+>>>     return "monkey2({})".format(data)
+>>> print(ub.repr2({1: float('nan'), 2: float('inf'), 3: 3.0}, nl=0))
 """
-from __future__ import absolute_import, division, print_function, unicode_literals
 import collections
-import sys
-# from typing import List, Callable, Type, Dict
 
 __all__ = ['repr2', 'FormatterExtensions']
 
-PY2 = sys.version_info[0] == 2
-
-if PY2:
-    import six
-    iteritems = six.iteritems
-    string_types = six.string_types
-    text_type = six.text_type
-else:
-    string_types = (str,)
-    text_type = str
-
-    def iteritems(d, **kw):
-        return d.items(**kw)
-
 
 def repr2(data, **kwargs):
     """
@@ -390,7 +381,7 @@ class FormatterExtensions(object):
             if isinstance(key, tuple):
                 for t in key:
                     self._type_registry[t] = func
-            if isinstance(key, string_types):
+            if isinstance(key, str):
                 self._typename_registry[key] = func
             else:
                 self._type_registry[key] = func
@@ -572,7 +563,7 @@ class FormatterExtensions(object):
             strvals = kwargs.get('sv', kwargs.get('strvals', False))
 
             if precision is None:
-                text = text_type(data)
+                text = str(data)
             else:
                 text = ('{:.%df}' % precision).format(data)
 
@@ -624,14 +615,8 @@ _FORMATTER_EXTENSIONS._lazy_queue.append(_lazy_init)
 def _format_object(val, **kwargs):
     stritems = kwargs.get('si', kwargs.get('stritems', False))
     strvals = stritems or kwargs.get('sv', kwargs.get('strvals', False))
-    base_valfunc = text_type if strvals else repr
-
+    base_valfunc = str if strvals else repr
     itemstr = base_valfunc(val)
-
-    # Remove unicode repr from python2 to agree with python3 output
-    if PY2 and isinstance(val, string_types):  # nocover
-        if itemstr.startswith(("u'", 'u"')):
-            itemstr = itemstr[1:]
     return itemstr
 
 
@@ -778,7 +763,7 @@ def _format_dict(dict_, **kwargs):
     itemsep = kwargs.get('itemsep', ' ')
 
     align = kwargs.get('align', False)
-    if align and not isinstance(align, string_types):
+    if align and not isinstance(align, str):
         default_kvsep = ': '
         if explicit:
             default_kvsep = '='
@@ -891,7 +876,7 @@ def _dict_itemstrs(dict_, **kwargs):
 
     def make_item_str(key, val):
         if explicit or kwargs.get('strkeys', default_strkeys):
-            key_str = text_type(key)
+            key_str = str(key)
         else:
             key_str = repr2(key, precision=precision, newlines=0)
 
@@ -920,7 +905,7 @@ def _dict_itemstrs(dict_, **kwargs):
             item_str = prefix + val_str
         return item_str, _leaf_info
 
-    items = list(iteritems(dict_))
+    items = list(dict_.items())
     _tups = [make_item_str(key, val) for (key, val) in items]
     itemstrs = [t[0] for t in _tups]
     max_height = max([t[1]['max_height'] for t in _tups]) if _tups else 0
@@ -932,7 +917,12 @@ def _dict_itemstrs(dict_, **kwargs):
     if sort is None:
         # if sort is None, force orderings on unordered collections like dicts,
         # but keep ordering of ordered collections like OrderedDicts.
-        sort = True
+        # NOTE: WE WANT TO CHANGE THIS TO FALSE BY DEFAULT.
+        # MIGHT REQUIRE DEPRECATING PYTHON 3.6 SUPPORT
+        sort = True  # LEGACY UBELT BEHAVIOR
+        # HOW TO WE INTRODUCE A BACKWARDS COMPATIBLE WAY TO MAKE THIS CHANGE?
+        # sort = False  # cannot make this change safely
+
     if isinstance(dict_, collections.OrderedDict):
         # never sort ordered dicts; they are perfect just the way they are!
         sort = False
@@ -1181,3 +1171,9 @@ def _align_lines(line_list, character='=', replchar=None, pos=0):
         else:
             new_lines.append(replchar.join(tup))
     return new_lines
+
+
+# Give the repr2 function itself a reference to the default extensions
+# register method so the user can modify them without accessing this module
+repr2.extensions = _FORMATTER_EXTENSIONS
+repr2.register = _FORMATTER_EXTENSIONS.register
