@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 Introduces the Executor class that wraps the standard ThreadPoolExecutor,
 ProcessPoolExecutor, and the new SerialExecutor with a common interface and a
@@ -41,7 +40,6 @@ Example:
     >>> print(ub.repr2(ti))
 
 """
-from __future__ import absolute_import, division, print_function, unicode_literals
 import concurrent.futures
 from concurrent.futures import as_completed
 
@@ -114,6 +112,10 @@ class SerialExecutor(object):
     """
     Implements the concurrent.futures API around a single-threaded backend
 
+    Notes:
+        When using the SerialExecutor, any timeout specified to the result will
+        be ignored.
+
     Example:
         >>> from ubelt.util_futures import SerialExecutor  # NOQA
         >>> import concurrent.futures
@@ -425,9 +427,45 @@ class JobPool(object):
                 progkw = {}
             job_iter = ub.ProgIter(
                 job_iter, desc=desc, total=len(self.jobs), **progkw)
+            self._prog = job_iter
         for job in job_iter:
             yield job
 
+    def join(self, **kwargs):
+        """
+        Like :method:`JobPool.as_completed`, but executes the `result` method
+        of each future and returns only after all processes are complete.
+        This allows for lower-boilerplate prototyping.
+
+        Args:
+            **kwargs: passed to :method:`JobPool.as_completed`
+
+        Returns:
+            List[Any]: list of results
+
+        Example:
+            >>> import ubelt as ub
+            >>> # We just want to try replacing our simple iterative algorithm
+            >>> # with the embarassingly parallel version
+            >>> arglist = list(zip(range(1000), range(1000)))
+            >>> func = ub.identity
+            >>> #
+            >>> # Original version
+            >>> for args in arglist:
+            >>>     func(*args)
+            >>> #
+            >>> # Potentially parallel version
+            >>> jobs = ub.JobPool(max_workers=0)
+            >>> for args in arglist:
+            >>>     jobs.submit(func, *args)
+            >>> _ = jobs.join(desc='running')
+        """
+        results = []
+        for job in self.as_completed(**kwargs):
+            result = job.result()
+            results.append(result)
+        return results
+
     def __iter__(self):
         """
         An alternative to as completed
