@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 r"""
 Wrappers around hashlib functions to generate hash signatures for common data.
 
@@ -30,8 +29,8 @@ Example:
     >>>     'param4': ('str', 4.2),
     >>> }.items()))
     >>> # hash_data can hash any ordered builtin object
-    >>> ub.hash_data(data, convert=False, hasher='sha512')
-    2ff39d0ecbf6ecc740ca7d...
+    >>> ub.hash_data(data, hasher='sha256')
+    0b101481e4b894ddf6de57...
 
 Example:
     >>> import ubelt as ub
@@ -55,13 +54,9 @@ Note:
     getting into the weeds of how we coerce technically non-hashable sequences
     into a hashable encoding.
 """
-from __future__ import absolute_import, division, print_function, unicode_literals
 import hashlib
-import sys
 import math
 from collections import OrderedDict
-# from typing import List, Callable, Type, Dict  # NOQA
-# we will use NoParam instead of None because None is a valid hashlen setting
 from ubelt.util_const import NoParam
 
 __all__ = ['hash_data', 'hash_file']
@@ -85,28 +80,9 @@ _ALPHABET_36 = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
 
 DEFAULT_ALPHABET = _ALPHABET_16
 
-PY2 = sys.version_info[0] == 2  # type: bool
 
-if PY2:
-    import six
-    from six.moves import zip
-    b = six.b
-    string_types = six.string_types
-    binary_type = six.binary_type
-    text_type = six.text_type
-    _stringlike = (basestring, bytes)  # NOQA
-    _intlike = (int, long)  # NOQA
-else:
-    # zip = zip  # hack for editor
-    from builtins import zip   # hack for editor / static analysis
-    six = None
-    def b(s):
-        return s.encode("latin-1")
-    binary_type = bytes
-    text_type = str
-    string_types = (str,)
-    _stringlike = (str, bytes)  # NOQA
-    _intlike = (int,)
+def b(s):
+    return s.encode("latin-1")
 
 # Sensible choices for default hashers are sha1, sha512, and xxh64.
 
@@ -130,88 +106,46 @@ DEFAULT_HASHER = hashlib.sha512  # most robust algo, but slower than others
 _COMPATIBLE_HASHABLE_SEQUENCE_TYPES_DEFAULT = True
 
 
-if PY2:
-    import codecs
-    # HASH = type(hashlib.sha1())  # python2 doesn't expose the hash type
+# Note: the Hasher refers to hashlib._hashlib.HASH
+# but this does not play well with type annotations
+# try:
+#     HASH = hashlib._hashlib.HASH
+# except AttributeError:  # nocover
+#     # Python seems to have been compiled without OpenSSL
+#     HASH = None
 
-    def _py2_to_bytes(int_, length, byteorder='big', signed=True):
-        """
-        Workaround for [PythonBug16580]_.
+def _int_to_bytes(int_):
+    r"""
+    Converts an integer into its byte representation
+    assumes int32 by default, but dynamically handles larger ints
 
-        Args:
-            length (int) : number of bytes (not bits)
+    Example:
+        >>> from ubelt.util_hash import _int_to_bytes, _bytes_to_int
+        >>> int_ = 1
+        >>> assert _bytes_to_int((_int_to_bytes(int_))) == int_
+        >>> assert _int_to_bytes(int_) == b'\x01'
+        >>> assert _bytes_to_int((_int_to_bytes(0))) == 0
+        >>> assert _bytes_to_int((_int_to_bytes(-1))) == -1
+        >>> assert _bytes_to_int((_int_to_bytes(-1000000))) == -1000000
+        >>> assert _bytes_to_int((_int_to_bytes(1000000))) == 1000000
+    """
+    bit_length = int_.bit_length() + 1
+    length = math.ceil(bit_length / 8.0)  # bytelength
+    bytes_ = int_.to_bytes(length, byteorder='big', signed=True)
+    return bytes_
 
-        References:
-            .. [PythonBug16580] https://bugs.python.org/issue16580
-        """
-        # convert nbytes to nbits
-        bit_width = length * 8
-        if int_ < 0:
-            complement = (1 << bit_width) + int_
-        else:
-            complement = int_
-        h = '%x' % complement
-        p = ('0' * (len(h) % 2) + h).zfill(length * 2)
-        s = p.decode('hex')
-        bytes_ = s if byteorder == 'big' else s[::-1]
-        return bytes_
 
-    def _int_to_bytes(int_):
-        bit_length = int_.bit_length() + 1
-        length = int(math.ceil(bit_length / 8.0))  # bytelength
-        bytes_ = _py2_to_bytes(int_, length, byteorder='big', signed=True)
-        return bytes_
+def _bytes_to_int(bytes_):
+    r"""
+    Converts a string of bytes into its integer representation (big-endian)
 
-    def _bytes_to_int(bytes_):
-        nbytes = len(bytes_)
-        nbits = nbytes * 8
-        comp = int(codecs.encode(bytes_, 'hex'), 16)
-        if comp > 1 << (nbits - 1):
-            int_ = comp - (1 << (nbits))
-        else:
-            int_ = comp
-        return int_
-else:
-    # Note: the Hasher refers to hashlib._hashlib.HASH
-    # but this does not play well with type annotations
-    # try:
-    #     HASH = hashlib._hashlib.HASH
-    # except AttributeError:  # nocover
-    #     # Python seems to have been compiled without OpenSSL
-    #     HASH = None
-
-    codecs = None
-    def _int_to_bytes(int_):
-        r"""
-        Converts an integer into its byte representation
-        assumes int32 by default, but dynamically handles larger ints
-
-        Example:
-            >>> from ubelt.util_hash import _int_to_bytes, _bytes_to_int
-            >>> int_ = 1
-            >>> assert _bytes_to_int((_int_to_bytes(int_))) == int_
-            >>> assert _int_to_bytes(int_) == b'\x01'
-            >>> assert _bytes_to_int((_int_to_bytes(0))) == 0
-            >>> assert _bytes_to_int((_int_to_bytes(-1))) == -1
-            >>> assert _bytes_to_int((_int_to_bytes(-1000000))) == -1000000
-            >>> assert _bytes_to_int((_int_to_bytes(1000000))) == 1000000
-        """
-        bit_length = int_.bit_length() + 1
-        length = math.ceil(bit_length / 8.0)  # bytelength
-        bytes_ = int_.to_bytes(length, byteorder='big', signed=True)
-        return bytes_
-
-    def _bytes_to_int(bytes_):
-        r"""
-        Converts a string of bytes into its integer representation (big-endian)
-
-        Example:
-            >>> bytes_ = b'\x01'
-            >>> assert _int_to_bytes((_bytes_to_int(bytes_))) == bytes_
-            >>> assert _bytes_to_int(bytes_) == 1
-        """
-        int_ = int.from_bytes(bytes_, 'big', signed=True)
-        return int_
+    Example:
+        >>> bytes_ = b'\x01'
+        >>> assert _int_to_bytes((_bytes_to_int(bytes_))) == bytes_
+        >>> assert _bytes_to_int(bytes_) == 1
+    """
+    int_ = int.from_bytes(bytes_, 'big', signed=True)
+    return int_
 
 
 class _Hashers(object):
@@ -287,12 +221,14 @@ class _Hashers(object):
             if self._lazy_queue:
                 self._evaluate_registration_queue()
 
-            if isinstance(hasher, string_types):
+            if isinstance(hasher, str):
                 hasher_ = self.aliases.get(hasher, hasher)
                 if hasher_ in self.algos:  # pragma: no cover
                     return self.algos[hasher_]
                 else:
+                    # TODO: provide pip install messages for known hashers.
                     raise KeyError('unknown hasher: {}'.format(hasher))
+
         return hasher
 
 _HASHERS = _Hashers()
@@ -368,29 +304,6 @@ def _rectify_base(base):
         return base
 
 
-def _rectify_hashlen(hashlen):  # nocover
-    """
-    Example:
-        >>> assert _rectify_hashlen(NoParam) is None
-        >>> assert _rectify_hashlen(8) == 8
-    """
-    if hashlen is NoParam:
-        return None
-    else:  # nocover
-        # import warnings
-        from ubelt._util_deprecated import schedule_deprecation2
-        schedule_deprecation2(
-            migration='Use slice syntax instead', name='hashlen', type='kwarg',
-            deprecate='0.9.6', remove='1.0.0',
-        )
-        # warnings.warn('Specifying hashlen is deprecated and will be removed. '
-        #               'Use slice syntax instead', DeprecationWarning)
-        if hashlen == 'default':  # nocover
-            return None
-        else:
-            return hashlen
-
-
 class HashableExtensions(object):
     """
     Helper class for managing non-primitive (e.g. numpy) hash types
@@ -404,6 +317,13 @@ class HashableExtensions(object):
         self.iterable_checks = []
         self._lazy_queue = []  # type: List[Callable]  # NOQA
 
+        # New singledispatch registry implementation
+        from functools import singledispatch
+        def _hash_dispatch(data):
+            raise NotImplementedError
+        _hash_dispatch.__is_base__ = True
+        self._hash_dispatch = singledispatch(_hash_dispatch)
+
     def register(self, hash_types):
         """
         Registers a function to generate a hash for data of the appropriate
@@ -439,6 +359,29 @@ class HashableExtensions(object):
             >>> my_instance = MyType(1)
             >>> ub.hash_data(my_instance, extensions=extensions)
 
+        Example:
+            >>> # xdoctest: +SKIP
+            >>> # Simple example
+            >>> import ubelt as ub
+            >>> ub.hash_data.register(pathlib.Path)(lambda x: (b'PATH', str))
+
+        Example:
+            >>> # Can specify more than one type when you register
+            >>> import ubelt as ub
+            >>> import pytest
+            >>> extensions = ub.util_hash.HashableExtensions()
+            >>> class Type1(object):
+            >>>     ...
+            >>> class Type2(object):
+            >>>     ...
+            >>> @extensions.register([Type1, Type2])
+            >>> def hash_my_type(data):
+            ...     return b'mytype', b'constant'
+            >>> inst1 = Type1()
+            >>> inst2 = Type2()
+            >>> ub.hash_data(inst1, extensions=extensions)
+            >>> ub.hash_data(inst2, extensions=extensions)
+
         Example:
             >>> # xdoctest: +SKIP
             >>> # Skip this doctest because we dont want tests to modify
@@ -459,24 +402,24 @@ class HashableExtensions(object):
             ...     return b'mytype', b(ub.hash_data(data.id))
             >>> my_instance = MyType(1)
             >>> ub.hash_data(my_instance)
+            >>> # New in ubelt 1.1.0: you can now do:
+            >>> # You can register your functions with ubelt's internal
+            >>> # hashable_extension registry.
+            >>> @ub.hash_data.register(MyType)
+            >>> def hash_my_type(data):
+            ...     return b'mytype', b(ub.hash_data(data.id))
+            >>> my_instance = MyType(1)
+            >>> ub.hash_data(my_instance)
         """
         # ensure iterable
         if not isinstance(hash_types, (list, tuple)):
             hash_types = [hash_types]
         def _decor_closure(hash_func):
             for hash_type in hash_types:
-                key = (hash_type.__module__, hash_type.__name__)
-                self.keyed_extensions[key] = (hash_type, hash_func)
+                self._hash_dispatch.register(hash_type)(hash_func)
             return hash_func
         return _decor_closure
 
-    def add_iterable_check(self, func):
-        """
-        Registers a function that detects when a type is iterable
-        """
-        self.iterable_checks.append(func)
-        return func
-
     def lookup(self, data):
         """
         Returns an appropriate function to hash ``data`` if one has been
@@ -490,7 +433,7 @@ class HashableExtensions(object):
             >>> import pytest
             >>> if not ub.modname_to_modpath('numpy'):
             ...     raise pytest.skip('numpy is optional')
-            >>> self = HashableExtensions()
+            >>> self = ub.util_hash.HashableExtensions()
             >>> self._register_numpy_extensions()
             >>> self._register_builtin_class_extensions()
 
@@ -515,35 +458,44 @@ class HashableExtensions(object):
             >>> import uuid
             >>> data = uuid.uuid4()
             >>> self.lookup(data)
+
+        Example:
+            >>> # xdoctest: +REQUIRES(module:numpy)
+            >>> import numpy as np
+            >>> import ubelt as ub
+            >>> self = ub.util_hash.HashableExtensions()
+            >>> self._register_numpy_extensions()
+            >>> self._register_builtin_class_extensions()
+            >>> #f1 = self.lookup(3)
+            >>> data = np.array([1, 2, 3])
+            >>> f2 = self.lookup(data)
+            >>> print(f2(data))
+            >>> data = np.uint8(3)
+            >>> f3 = self.lookup(data)
+            >>> print(f3(data))
         """
         # Evaluate the lazy queue if anything is in it
         if self._lazy_queue:
             for func in self._lazy_queue:
                 func()
             self._lazy_queue = []
-
-        # Maybe try using functools.singledispatch instead?
-        # First try O(1) lookup
         query_hash_type = data.__class__
-        key = (query_hash_type.__module__, query_hash_type.__name__)
-        try:
-            hash_type, hash_func = self.keyed_extensions[key]
-        except KeyError:
-            # if issubclass(query_hash_type, dict):
-            #     # TODO: In Python 3.7+ dicts are ordered by default, so
-            #     # perhaps we should allow hashing them by default
-            #     import warnings
-            #     warnings.warn(
-            #         'It looks like you are trying to hash an unordered dict. '
-            #         'By default this is not allowed, but if you REALLY need '
-            #         'to do this, then call '
-            #         'ubelt.util_hash._HASHABLE_EXTENSIONS._register_agressive_extensions() '
-            #         'beforehand')
+        # TODO: recognize some special dunder method instead
+        # of strictly using this registry.
+        hash_func = self._hash_dispatch.dispatch(query_hash_type)
+        if getattr(hash_func, '__is_base__', False):
             raise TypeError(
                 'No registered hash func for hashable type={!r}'.format(
                     query_hash_type))
         return hash_func
 
+    def add_iterable_check(self, func):
+        """
+        Registers a function that detects when a type is iterable
+        """
+        self.iterable_checks.append(func)
+        return func
+
     def _register_numpy_extensions(self):
         """
         Registers custom functions to hash numpy data structures.
@@ -552,9 +504,6 @@ class HashableExtensions(object):
         """
         # system checks
         import numpy as np
-        numpy_floating_types = (np.float16, np.float32, np.float64)
-        if hasattr(np, 'float128'):  # nocover
-            numpy_floating_types = numpy_floating_types + (np.float128,)
 
         @self.add_iterable_check
         def is_object_ndarray(data):
@@ -595,15 +544,6 @@ class HashableExtensions(object):
             prefix = b'NDARR'
             return prefix, hashable
 
-        @self.register((np.int64, np.int32, np.int16, np.int8,
-                        np.uint64, np.uint32, np.uint16, np.uint8))
-        def _convert_numpy_int(data):
-            return _convert_to_hashable(int(data), extensions=self)
-
-        @self.register(numpy_floating_types)
-        def _convert_numpy_float(data):
-            return _convert_to_hashable(float(data), extensions=self)
-
         @self.register(np.random.RandomState)
         def _convert_numpy_random_state(data):
             """
@@ -690,8 +630,34 @@ class HashableExtensions(object):
             >>> print(ub.hash_data({3, 1, '2'})[0:8])
             742ae82d
             742ae82d
+
+        Example:
+            >>> import ubelt as ub
+            >>> assert ub.hash_data(slice(None)).startswith('0178e55a247d09ad282dc2e44f5388f477')
+
+        Example:
+            >>> import ubelt as ub
+            >>> print(ub.hash_data(ub.Path('foo'))[0:8])
+            >>> print(ub.hash_data('foo')[0:8])
+            >>> print(ub.hash_data(ub.Path('foo'), types=True)[0:8])
+            >>> print(ub.hash_data('foo', types=True)[0:8])
+            f7fbba6e
+            f7fbba6e
+            cc21b9fa
+            bd1cabd0
         """
         import uuid
+        import pathlib
+        import numbers
+
+        @self.register(numbers.Integral)
+        def _convert_numpy_int(data):
+            return _convert_to_hashable(int(data), extensions=self)
+
+        @self.register(numbers.Real)
+        def _convert_numpy_float(data):
+            return _convert_to_hashable(float(data), extensions=self)
+
         @self.register(uuid.UUID)
         def _convert_uuid(data):
             hashable = data.bytes
@@ -744,6 +710,22 @@ class HashableExtensions(object):
             prefix = b'ODICT'
             return prefix, hashable
 
+        @self.register(slice)
+        def _convert_slice(data):
+            """
+            Currently ordered dictionaries are considered separately from
+            regular dictionaries. I'm not sure what the right thing to do is.
+            """
+            # See: [util_hash.Note.1]
+            hashable = b''.join(_hashable_sequence(
+                [data.start, data.stop, data.step], extensions=self,
+                types=_COMPATIBLE_HASHABLE_SEQUENCE_TYPES_DEFAULT))
+            prefix = b'SLICE'
+            return prefix, hashable
+
+        self.register(pathlib.Path)(lambda x: (b'PATH', str(x).encode('utf-8')))
+        # other data structures
+
     def _register_agressive_extensions(self):  # nocover
         """
         Extensions that might be desired, but we do not enable them by default
@@ -830,12 +812,12 @@ def _convert_to_hashable(data, types=True, extensions=None):
         TypeError : if data has no registered hash methods
 
     Example:
-        >>> from ubelt.util_hash import _convert_to_hashable, _intlike
+        >>> from ubelt.util_hash import _convert_to_hashable
         >>> assert _convert_to_hashable(None) == (b'NULL', b'NONE')
         >>> assert _convert_to_hashable('string') == (b'TXT', b'string')
         >>> assert _convert_to_hashable(1) == (b'INT', b'\x01')
         >>> assert _convert_to_hashable(1.0) == (b'FLT', b'\x01/\x01')
-        >>> assert _convert_to_hashable(_intlike[-1](1)) == (b'INT', b'\x01')
+        >>> assert _convert_to_hashable(int(1)) == (b'INT', b'\x01')
         >>> import uuid
         >>> data = uuid.UUID('7e9d206b-dc02-4240-8bdb-fffe858121d0')
         >>> assert _convert_to_hashable(data) == (b'UUID', b'~\x9d k\xdc\x02B@\x8b\xdb\xff\xfe\x85\x81!\xd0')
@@ -850,14 +832,14 @@ def _convert_to_hashable(data, types=True, extensions=None):
     if data is None:
         hashable = b'NONE'
         prefix = b'NULL'
-    elif isinstance(data, binary_type):
+    elif isinstance(data, bytes):
         hashable = data
         prefix = b'TXT'
-    elif isinstance(data, text_type):
+    elif isinstance(data, str):
         # convert unicode into bytes
         hashable = data.encode('utf-8')
         prefix = b'TXT'
-    elif isinstance(data, _intlike):
+    elif isinstance(data, int):
         # warnings.warn('Hashing ints is slow, numpy is preferred')
         hashable = _int_to_bytes(data)
         # hashable = data.to_bytes(8, byteorder='big')
@@ -1004,19 +986,19 @@ def _convert_hexstr_base(hexstr, base):
     return newbase_str
 
 
-def _digest_hasher(hasher, hashlen, base):
+def _digest_hasher(hasher, base):
     """ counterpart to _update_hasher """
     # Get a 128 character hex string
     hex_text = hasher.hexdigest()
     # Shorten length of string (by increasing base)
     base_text = _convert_hexstr_base(hex_text, base)
     # Truncate
-    text = base_text[:hashlen]
+    text = base_text
     return text
 
 
-def hash_data(data, hasher=NoParam, base=NoParam, types=False,
-              hashlen=NoParam, convert=False, extensions=None):
+def hash_data(data, hasher=NoParam, base=NoParam, types=False, convert=False,
+              extensions=None):
     """
     Get a unique hash depending on the state of the data.
 
@@ -1038,10 +1020,6 @@ def hash_data(data, hasher=NoParam, base=NoParam, types=False,
             If True data types are included in the hash, otherwise only the raw
             data is hashed. Defaults to False.
 
-        hashlen (int):
-            Maximum number of symbols in the returned hash. If not specified,
-            all are returned.  DEPRECATED. Use slice syntax instead.
-
         convert (bool, default=True):
             if True, try and convert the data to json an the json is hashed
             instead. This can improve runtime in some instances, however the
@@ -1075,7 +1053,7 @@ def hash_data(data, hasher=NoParam, base=NoParam, types=False,
         >>> print(ub.hash_data([1, 2, (3, '4')], base='abc',  hasher='sha512')[:32])
         hsrgqvfiuxvvhcdnypivhhthmrolkzej
     """
-    if convert and not isinstance(data, string_types):  # nocover
+    if convert and not isinstance(data, str):  # nocover
         import json
         try:
             data = json.dumps(data)
@@ -1085,17 +1063,16 @@ def hash_data(data, hasher=NoParam, base=NoParam, types=False,
             pass
 
     base = _rectify_base(base)
-    hashlen = _rectify_hashlen(hashlen)
     hasher = _rectify_hasher(hasher)()
     # Feed the data into the hasher
     _update_hasher(hasher, data, types=types, extensions=extensions)
     # Get the hashed representation
-    text = _digest_hasher(hasher, hashlen, base)
+    text = _digest_hasher(hasher, base)
     return text
 
 
-def hash_file(fpath, blocksize=1048576, stride=1, maxbytes=None, hasher=NoParam,
-              hashlen=NoParam, base=NoParam):
+def hash_file(fpath, blocksize=1048576, stride=1, maxbytes=None,
+              hasher=NoParam, base=NoParam):
     """
     Hashes the data in a file on disk.
 
@@ -1128,10 +1105,6 @@ def hash_file(fpath, blocksize=1048576, stride=1, maxbytes=None, hasher=NoParam,
 
             TODO: add logic such that you can update an existing hasher
 
-        hashlen (int):
-            maximum number of symbols in the returned hash. If not specified,
-            all are returned. DEPRECATED. DO NOT USE.
-
         base (List[str] | str, default='hex'):
             list of symbols or shorthand key.
             Valid keys are 'abc', 'hex', and 'dec'.
@@ -1148,21 +1121,18 @@ def hash_file(fpath, blocksize=1048576, stride=1, maxbytes=None, hasher=NoParam,
     Example:
         >>> import ubelt as ub
         >>> from os.path import join
-        >>> fpath = join(ub.ensure_app_cache_dir('ubelt'), 'tmp.txt')
-        >>> ub.writeto(fpath, 'foobar')
+        >>> dpath = ub.Path.appdir('ubelt/tests/test-hash').ensuredir()
+        >>> fpath = dpath / 'tmp1.txt'
+        >>> fpath.write_text('foobar')
         >>> print(ub.hash_file(fpath, hasher='sha1', base='hex'))
         8843d7f92416211de9ebb963ff4ce28125932878
 
     Example:
         >>> import ubelt as ub
-        >>> from os.path import join
-        >>> fpath = join(ub.ensure_app_cache_dir('ubelt'), 'tmp.txt')
-        >>> ub.writeto(fpath, 'foobar')
-        >>> print(ub.hash_file(fpath, hasher='sha1', base='hex', maxbytes=1000))
-        8843d7f92416211de9ebb963ff4ce28125932878
-
+        >>> dpath = ub.Path.appdir('ubelt/tests/test-hash').ensuredir()
+        >>> fpath = dpath / 'tmp2.txt'
         >>> # We have the ability to only hash at most ``maxbytes`` in a file
-        >>> ub.writeto(fpath, 'abcdefghijklmnop')
+        >>> fpath.write_text('abcdefghijklmnop')
         >>> h0 = ub.hash_file(fpath, hasher='sha1', base='hex', maxbytes=11, blocksize=3)
         >>> h1 = ub.hash_file(fpath, hasher='sha1', base='hex', maxbytes=32, blocksize=3)
         >>> h2 = ub.hash_file(fpath, hasher='sha1', base='hex', maxbytes=32, blocksize=32)
@@ -1184,7 +1154,8 @@ def hash_file(fpath, blocksize=1048576, stride=1, maxbytes=None, hasher=NoParam,
     Example:
         >>> import ubelt as ub
         >>> from os.path import join
-        >>> fpath = ub.touch(join(ub.ensure_app_cache_dir('ubelt'), 'empty_file'))
+        >>> dpath = ub.ensure_app_cache_dir('ubelt/tests/test-hash')
+        >>> fpath = ub.touch(join(dpath, 'empty_file'))
         >>> # Test that the output is the same as sha1sum executable
         >>> if ub.find_exe('sha1sum'):
         >>>     want = ub.cmd(['sha1sum', fpath], verbose=2)['out'].split(' ')[0]
@@ -1207,7 +1178,6 @@ def hash_file(fpath, blocksize=1048576, stride=1, maxbytes=None, hasher=NoParam,
         >>>     assert want.endswith(got)
     """
     base = _rectify_base(base)
-    hashlen = _rectify_hashlen(hashlen)
     hasher = _rectify_hasher(hasher)()
     with open(fpath, 'rb') as file:
         buf = file.read(blocksize)
@@ -1245,5 +1215,11 @@ def hash_file(fpath, blocksize=1048576, stride=1, maxbytes=None, hasher=NoParam,
                         buf = file.read(blocksize)
 
     # Get the hashed representation
-    text = _digest_hasher(hasher, hashlen, base)
+    text = _digest_hasher(hasher, base)
     return text
+
+
+# Give the hash_data function itself a reference to the default extensions
+# register method so the user can modify them without accessing this module
+hash_data.extensions = _HASHABLE_EXTENSIONS
+hash_data.register = _HASHABLE_EXTENSIONS.register
