@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 Functions for reading and writing files on disk.
 
@@ -10,7 +9,6 @@ written and read is unicode text.
 throw an error if the file or directory does not exist. It also contains
 workarounds for win32 issues with :mod:`shutil`.
 """
-from __future__ import absolute_import, division, print_function, unicode_literals
 import sys
 import os
 from os.path import exists
@@ -21,9 +19,6 @@ __all__ = [
 ]
 
 
-PY2 = sys.version_info[0] == 2
-
-
 def writeto(fpath, to_write, aslines=False, verbose=None):
     r"""
     Writes (utf8) text to a file.
@@ -40,17 +35,23 @@ def writeto(fpath, to_write, aslines=False, verbose=None):
         After 2020-01-01, we may consider deprecating the function.
 
         NOTE: In PyPy ``open(<fpath>).write(<to_write>)`` does not work. See
-        `https://pypy.org/compat.html`. This is a strong argument
-        for keeping this function.
+        `https://pypy.org/compat.html`. This is an argument for keeping this
+        function.
+
+        NOTE: With modern versions of Python, it is generally recommened to use
+        :func:`pathlib.Path.write_text` instead. Although there does seem to be
+        some corner case this handles better on win32, so maybe useful?
 
     Example:
         >>> import ubelt as ub
+        >>> import os
+        >>> from os.path import exists
         >>> dpath = ub.ensure_app_cache_dir('ubelt')
         >>> fpath = dpath + '/' + 'testwrite.txt'
         >>> if exists(fpath):
         >>>     os.remove(fpath)
         >>> to_write = 'utf-8 symbols Δ, Й, ק, م, ๗, あ, 叶, 葉, and 말.'
-        >>> writeto(fpath, to_write)
+        >>> ub.writeto(fpath, to_write)
         >>> read_ = ub.readfrom(fpath)
         >>> print('read_    = ' + read_)
         >>> print('to_write = ' + to_write)
@@ -58,16 +59,28 @@ def writeto(fpath, to_write, aslines=False, verbose=None):
 
     Example:
         >>> import ubelt as ub
+        >>> import os
+        >>> from os.path import exists
         >>> dpath = ub.ensure_app_cache_dir('ubelt')
         >>> fpath = dpath + '/' + 'testwrite2.txt'
         >>> if exists(fpath):
         >>>     os.remove(fpath)
         >>> to_write = ['a\n', 'b\n', 'c\n', 'd\n']
-        >>> writeto(fpath, to_write, aslines=True)
+        >>> ub.writeto(fpath, to_write, aslines=True)
         >>> read_ = ub.readfrom(fpath, aslines=True)
         >>> print('read_    = {}'.format(read_))
         >>> print('to_write = {}'.format(to_write))
         >>> assert read_ == to_write
+
+    Example:
+        >>> # With modern Python, use pathlib.Path (or ub.Path) instead
+        >>> import ubelt as ub
+        >>> dpath = ub.Path.appdir('ubelt/tests/io').ensuredir()
+        >>> fpath = (dpath / 'test_file.txt').delete()
+        >>> to_write = 'utf-8 symbols Δ, Й, ק, م, ๗, あ, 叶, 葉, and 말.'
+        >>> ub.writeto(fpath, to_write)
+        >>> fpath.write_bytes(to_write.encode('utf8'))
+        >>> assert fpath.read_bytes().decode('utf8') == to_write
     """
     if verbose:
         print('Writing to text file: %r ' % (fpath,))
@@ -82,15 +95,9 @@ def writeto(fpath, to_write, aslines=False, verbose=None):
             file.write(bytes)
 
 
-if PY2:  # nocover
-    def _ensure_bytes(text):
-        if isinstance(text, unicode):  # NOQA
-            text = text.encode('utf8')
-        return text
-else:
-    def _ensure_bytes(text):
-        """ ensures text is in a suitable format for writing """
-        return text.encode('utf8')
+def _ensure_bytes(text):
+    """ ensures text is in a suitable format for writing """
+    return text.encode('utf8')
 
 
 def readfrom(fpath, aslines=False, errors='replace', verbose=None):
@@ -98,7 +105,7 @@ def readfrom(fpath, aslines=False, errors='replace', verbose=None):
     Reads (utf8) text from a file.
 
     Note:
-        You probably should use ``open(<fpath>).read()`` instead.
+        You probably should use ``ub.Path(<fpath>).read_text()`` instead.
         This function exists as a convenience for writing in Python2. After
         2020-01-01, we may consider deprecating the function.
 
@@ -162,14 +169,10 @@ def touch(fpath, mode=0o666, dir_fd=None, verbose=0, **kwargs):
     """
     if verbose:
         print('Touching file {}'.format(fpath))
-    if PY2:  # nocover
-        with open(fpath, 'a'):
-            os.utime(fpath, None)
-    else:
-        flags = os.O_CREAT | os.O_APPEND
-        with os.fdopen(os.open(fpath, flags=flags, mode=mode, dir_fd=dir_fd)) as f:
-            os.utime(f.fileno() if os.utime in os.supports_fd else fpath,
-                     dir_fd=None if os.supports_fd else dir_fd, **kwargs)
+    flags = os.O_CREAT | os.O_APPEND
+    with os.fdopen(os.open(fpath, flags=flags, mode=mode, dir_fd=dir_fd)) as f:
+        os.utime(f.fileno() if os.utime in os.supports_fd else fpath,
+                 dir_fd=None if os.supports_fd else dir_fd, **kwargs)
     return fpath
 
 
